package com.bjlt.spider.util;

import com.luyn.date.DatePattren;
import com.luyn.date.DateUtil;
import org.springframework.beans.factory.annotation.Value;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * 日志输出工具 <br/><br/>
 *
 * 可以输出到控制台和指定的文件中, 分为4个级别, 由低到高分别为: debug, info, warn, error
 *
 * <br/><br/>
 *
 * 输出级别:
 *
 * <ul>
 *     <li> debug: 输出 debug, info, warn, error </li>
 *     <li> info: 输出 info, warn, error </li>
 *     <li> warn: 输出 warn, error </li>
 *     <li> error: 输出 error </li>
 * </ul>
 *
 * 默认为 info 输出级别
 *
 * <p/>
 *
 * Demo:
 *
 * <pre>{@code
 *     // (可选) 设置日志输出级别, 默认为 INFO 级别
 *     LogUtils.setLogOutLevel(LogUtils.Level.DEBUG);
 *
 *     // (可选) 设置日志输出文件(追加到文件尾部)
 *     LogUtils.setLogOutFile(new File("MyLog.log"));
 *
 *     // (可选) 设置日志输出位置(是否输出到控制台 和 是否输出到文件), 默认只输出到控制台, 不输出到文件
 *     LogUtils.setLogOutTarget(true, true);
 *
 *     // 输出日志
 *     LogUtils.debug("TAG", "The debug log.");
 *     LogUtils.info("TAG", "The info log.");
 *     LogUtils.warn("TAG", "The warn log.");
 *     LogUtils.error("TAG", "The error log.");
 * }</pre>
 *
 * @author xietansheng
 */
public class LogUtils {
    @Value("${ErrorLog}")
    private String errorLog;

//
//    /** 每条 Log 的 tag 输出的最大长度, 超过部分将被截断 */
//    private static final int TAG_MAX_LENGTH = 20;
//
//    /** 每条 Log 的 message 输出的最大长度, 超过部分将被截断 */
//    private static final int MESSAGE_MAX_LENGTH = 10240;

    /** 日期前缀格式化 */
    private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy/MM/dd HH/mm/ss");

    /** 日志当前的输出级别, 默认为 INFO 级别 */
    private static Level logOutLevel = Level.INFO;

    /** 是否输出到控制台, 默认输出 */
    private static boolean isOutToConsole = false;

    /** 是否输出到文件 */
    private static boolean isOutToFile = true;

    /**是否进行打包*/
    private static boolean isZip=true;
    /** 日志输出文件, 追加到文件尾 */
    private static File logOutFile;

    /** 日志文件输出流, 追加到文件尾  */
    private static RandomAccessFile logOutFileStream;

    public static void setLogOutLevel(Level currentLevel) {
        if (currentLevel == null) {
            currentLevel = Level.INFO;
        }
        LogUtils.logOutLevel = currentLevel;
    }

    public static synchronized void setLogOutFile(String filePath, String datetime,String type) throws IOException {

        //日期文件夹下创建日志文件 20180204143
        File filepath = new File(filePath+"/"+datetime.substring(0,8)+type);
        if (!filepath.isDirectory()){
            filepath.mkdirs();
        }
        File file1 = new File(filepath.getPath()+"/"+datetime.substring(0,11)+".txt");

        if (file1.exists()){
            //追加模式
            LogUtils.logOutFile = file1;
        }else {
            file1.createNewFile();
            LogUtils.logOutFile = file1;
        }
        if (logOutFileStream != null) {
            closeStream(logOutFileStream);
            logOutFileStream = null;
        }
        if (LogUtils.logOutFile != null) {
            try {
                logOutFileStream = new RandomAccessFile(LogUtils.logOutFile, "rw");
                logOutFileStream.seek(LogUtils.logOutFile.length());
            } catch (IOException e) {
                closeStream(logOutFileStream);
                logOutFileStream = null;
                throw e;
            }
        }
        //打包昨天的文件夹
        File file = new File(filePath+DateUtil.getFormatDate(getDateBefore(new Date(),1),DatePattren.DATA_FORMAT_14).substring(0,8)+"ORDER成功");
        if (file.exists()&& file.isDirectory()){
            fileToZip(file.getPath(),filePath,DateUtil.getFormatDate(getDateBefore(new Date(),1),DatePattren.DATA_FORMAT_14).substring(0,8)+"ORDER成功");
            // 删除文件夹
            File[] files2 = file.listFiles();//声明目录下所有的文件 files[];
            for (int i = 0;i < files2.length;i ++) {//遍历目录下所有的文件
                boolean delete = files2[i].delete();
                System.out.println(delete);
            }
            boolean delete = file.delete();
            System.out.println(delete);
        }

        File fileError = new File(filePath+DateUtil.getFormatDate(getDateBefore(new Date(),1),DatePattren.DATA_FORMAT_14).substring(0,8)+"ORDER失败");
        if (fileError.exists()&& fileError.isDirectory()){
            fileToZip(fileError.getPath(),filePath,DateUtil.getFormatDate(getDateBefore(new Date(),1),DatePattren.DATA_FORMAT_14).substring(0,8)+"ORDER失败");
            // 删除文件夹
            File[] files2 = fileError.listFiles();//声明目录下所有的文件 files[];
            for (int i = 0;i < files2.length;i ++) {//遍历目录下所有的文件
                boolean delete = files2[i].delete();
                System.out.println(delete);
            }
            boolean delete = fileError.delete();
            System.out.println(delete);
        }
        File fileve = new File(filePath+DateUtil.getFormatDate(getDateBefore(new Date(),1),DatePattren.DATA_FORMAT_14).substring(0,8)+"VERIFY失败");
        if (fileve.exists()&& fileve.isDirectory()){
            fileToZip(fileve.getPath(),filePath,DateUtil.getFormatDate(getDateBefore(new Date(),1),DatePattren.DATA_FORMAT_14).substring(0,8)+"VERIFY失败");
            // 删除文件夹
            File[] files2 = fileve.listFiles();//声明目录下所有的文件 files[];
            for (int i = 0;i < files2.length;i ++) {//遍历目录下所有的文件
                files2[i].delete();
            }
            fileve.delete();
        }
        File filevs = new File(filePath+DateUtil.getFormatDate(getDateBefore(new Date(),1),DatePattren.DATA_FORMAT_14).substring(0,8)+"VERIFY成功");
        if (filevs.exists()&& filevs.isDirectory()){
            fileToZip(filevs.getPath(),filePath,DateUtil.getFormatDate(getDateBefore(new Date(),1),DatePattren.DATA_FORMAT_14).substring(0,8)+"VERIFY成功");
            // 删除文件夹
            File[] files2 = filevs.listFiles();//声明目录下所有的文件 files[];
            for (int i = 0;i < files2.length;i ++) {//遍历目录下所有的文件
                files2[i].delete();
            }
            filevs.delete();
        }


    }

    public static void setLogOutTarget(boolean isOutToConsole, boolean isOutToFile) {
        LogUtils.isOutToConsole = isOutToConsole;
        LogUtils.isOutToFile = isOutToFile;
    }

    public static void debug(String tag, String message) {
        printLog(Level.DEBUG, tag, message, false);
    }

    public static void info(String tag, String message) {
        printLog(Level.INFO, tag, message, false);
    }

    public static void warn(String tag, String message) {
        printLog(Level.WARN, tag, message, false);
    }

    public static void error(String tag, String message) {
        printLog(Level.ERROR, tag, message, true);
    }

    public static void error(String tag, Exception e) {
        if (e == null) {
            error(tag, (String) null);
            return;
        }

        PrintStream printOut = null;

        try {
            ByteArrayOutputStream bytesBufOut = new ByteArrayOutputStream();
            printOut = new PrintStream(bytesBufOut);
            e.printStackTrace(printOut);
            printOut.flush();
            error(tag, new String(bytesBufOut.toByteArray(), "UTF-8"));

        } catch (Exception e1) {
            e1.printStackTrace();

        } finally {
            closeStream(printOut);
        }
    }

    private static void printLog(Level level, String tag, String message, boolean isOutToErr) {
        if (level.getLevelValue() >= logOutLevel.getLevelValue()) {
//            String log = DATE_FORMAT.format(new Date()) +
//                    " " +
//                    level.getTag() +
//                    "/" +
//                    checkTextLengthLimit(tag, TAG_MAX_LENGTH) +
//                    ": " +
//                    checkTextLengthLimit(message, MESSAGE_MAX_LENGTH);

            String log = DATE_FORMAT.format(new Date()) +
                    " " +
                    level.getTag() +
                    "/" +
                    tag +
                    ": " +
                    message;

            if (isOutToConsole) {
                outLogToConsole(isOutToErr, log);
            }
            if (isOutToFile) {
                outLogToFile(log);
            }
        }
    }

    private static void outLogToConsole(boolean isOutToErr, String log) {
        if (isOutToErr) {
            // System.err 和 System.out 是两个不同的输出流通道, 如果极短时间内连
            // 续输出 log 到 err 和 out, 控制台上的打印顺序可能会不完全按时序打印.
            System.err.println(log);
        } else {
            System.out.println(log);
        }
    }

    private static synchronized void outLogToFile(String log) {
        if (logOutFileStream != null) {
            try {
                logOutFileStream.write((log + "\r\n").getBytes("UTF-8"));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private static String checkTextLengthLimit(String text, int maxLength) {
        if ((text != null) && (text.length() >  maxLength)) {
            text = text.substring(0, maxLength - 3) + "...";
        }
        return text;
    }

    private static void closeStream(Closeable stream) {
        if (stream != null) {
            try {
                stream.close();
            } catch (Exception e) {
                // nothing
            }
        }
    }

    public static enum Level {
        DEBUG("D", 1), INFO("I", 2), WARN("W", 3), ERROR("E", 4);

        private String tag;




        private int levelValue;

        private Level(String tag, int levelValue) {
            this.tag = tag;
            this.levelValue = levelValue;
        }

        public String getTag() {
            return tag;
        }

        public int getLevelValue() {
            return levelValue;
        }
    }
    public static boolean fileToZip(String sourceFilePath,String zipFilePath,String fileName){
        boolean flag = false;
        File sourceFile = new File(sourceFilePath);
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        FileOutputStream fos = null;
        ZipOutputStream zos = null;

        if(sourceFile.exists() == false){
            System.out.println("待压缩的文件目录："+sourceFilePath+"不存在.");
        }else{
            try {
                File zipFile = new File(zipFilePath + "/" + fileName +".zip");
                if(zipFile.exists()){
                    System.out.println(zipFilePath + "目录下存在名字为:" + fileName +".zip" +"打包文件.");
                }else{
                    File[] sourceFiles = sourceFile.listFiles();
                    if(null == sourceFiles || sourceFiles.length<1){
                        System.out.println("待压缩的文件目录：" + sourceFilePath + "里面不存在文件，无需压缩.");
                    }else{
                        fos = new FileOutputStream(zipFile);
                        zos = new ZipOutputStream(new BufferedOutputStream(fos));
                        byte[] bufs = new byte[1024*10];
                        for(int i=0;i<sourceFiles.length;i++){
                            //创建ZIP实体，并添加进压缩包
                            ZipEntry zipEntry = new ZipEntry(sourceFiles[i].getName());
                            zos.putNextEntry(zipEntry);
                            //读取待压缩的文件并写进压缩包里
                            fis = new FileInputStream(sourceFiles[i]);
                            bis = new BufferedInputStream(fis, 1024*10);
                            int read = 0;
                            while((read=bis.read(bufs, 0, 1024*10)) != -1){
                                zos.write(bufs,0,read);
                            }
                        }
                        flag = true;
                    }
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
                throw new RuntimeException(e);
            } catch (IOException e) {
                e.printStackTrace();
                throw new RuntimeException(e);
            } finally{
                //关闭流
                try {
                    if(null != bis) bis.close();
                    if(null != zos) zos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                    throw new RuntimeException(e);
                }
            }
        }
        return flag;
    }

    private static void compress(File sourceFile, ZipOutputStream zos,
                                 boolean KeepDirStructure) throws Exception {
            File[] listFiles = sourceFile.listFiles();
                for (File file : listFiles) {
                    // 判断是否需要保留原来的文件结构
                        compress(file, zos, KeepDirStructure);
                }
            }

    public static Date getDateBefore(Date d, int day) {
        Calendar now = Calendar.getInstance();
        now.setTime(d);
        now.set(Calendar.DATE, now.get(Calendar.DATE) - day);
        return now.getTime();
    }
//
    public static void main(String[] args) {
        try {
            LogUtils.setLogOutFile("C:/Users/admin/Desktop/errorLog/", DateUtil.getFormatDate(new Date(), DatePattren.DATA_FORMAT_14),"验价失败");
            LogUtils.error("11111123123","12312312");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }



}